/**************************************************************************
*  Copyright (c) 2013-2015 by Michael Fischer.
*  All rights reserved.
*
*  Redistribution and use in source and binary forms, with or without 
*  modification, are permitted provided that the following conditions 
*  are met:
*  
*  1. Redistributions of source code must retain the above copyright 
*     notice, this list of conditions and the following disclaimer.
*  2. Redistributions in binary form must reproduce the above copyright
*     notice, this list of conditions and the following disclaimer in the 
*     documentation and/or other materials provided with the distribution.
*  3. Neither the name of the author nor the names of its contributors may 
*     be used to endorse or promote products derived from this software 
*     without specific prior written permission.
*
*  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
*  "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
*  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS 
*  FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL 
*  THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
*  INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
*  BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS 
*  OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED 
*  AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, 
*  OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF 
*  THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF 
*  SUCH DAMAGE.
*
***************************************************************************
*  History:
*
*  19.08.2013  mifi  First Version, tested with a STM3240G-EVAL board.
*  21.08.2013  mifi  Added output of IP information.
*  21.08.2013  mifi  Changed for ChibiOS.
*  18.04.2014  mifi  Update to ChibiOS (v2.6.3) and lwIP (v1.4.1).
*  08.06.2015  mifi  Update to ChibiOS (v2.6.8) and corrected HardwareInit. 
**************************************************************************/
#define __MAIN_C__

/*=======================================================================*/
/*  Includes                                                             */
/*=======================================================================*/
#include <stdio.h>
#include <string.h>

#include "project.h"
#include "ipstack.h"
#include "iperf.h"
#include "pro\uhttp\mediatypes.h"
#include "pro\uhttp\modules\mod_redir.h"


/*
 * Check for CrossWorks for ARM, CTL is used.
 */
#ifndef __CROSSWORKS_ARM
   Error: Wrong compiler used;
#endif   


/* 
 * Check HSE value, must be 12MHz 
 */
#if (HSE_VALUE != 12000000)
   Error: HSE_VALUE must defined to 12000000 by the preprocessor;
#endif   

/*=======================================================================*/
/*  All Structures and Common Constants                                  */
/*=======================================================================*/

/*
 * Led LD1 supported.
 */
#define LD1_ON()  GPIOC->BSRRH = (1<<13)
#define LD1_OFF() GPIOC->BSRRL = (1<<13)

/*=======================================================================*/
/*  Definition of all global Data                                        */
/*=======================================================================*/

/*=======================================================================*/
/*  Definition of all local Data                                         */
/*=======================================================================*/

/* 
 * Stack and task definition.
 */
static WORKING_AREA(task_led_stack, TASK_LED_STK_SIZE);
static WORKING_AREA(task_start_stack, TASK_START_STK_SIZE);
static WORKING_AREA(task_http_stack, TASK_IP_HTTP_STK_SIZE); 

/*=======================================================================*/
/*  Definition of all local Procedures                                   */
/*=======================================================================*/
/*************************************************************************/
/*  USART3Init                                                           */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void USART3Init (void)
{
   USART_InitTypeDef USART_InitStruct;
   GPIO_InitTypeDef  GPIO_InitStruct;

   /* 
    * Use USART3, PC10 (TX) and PC11 (RX)  
    */
   
   /* Turn on GPIOC clock */
   RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);
   
   /* Turn on USART3 clock */
   RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);   

   /* Connect PC10 to USART3_TX, AF7 */
   GPIO_PinAFConfig(GPIOC, GPIO_PinSource10, GPIO_AF_USART3);

   /* Connect PC11 to USART3_RX, AF7 */
   GPIO_PinAFConfig(GPIOC, GPIO_PinSource11, GPIO_AF_USART3);

   /* Configure USART3_TX as alternate function */
   GPIO_StructInit(&GPIO_InitStruct);
   GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
   GPIO_InitStruct.GPIO_PuPd  = GPIO_PuPd_UP;
   GPIO_InitStruct.GPIO_Mode  = GPIO_Mode_AF;
   GPIO_InitStruct.GPIO_Pin   = GPIO_Pin_10;
   GPIO_InitStruct.GPIO_Speed = GPIO_Speed_50MHz;
   GPIO_Init(GPIOC, &GPIO_InitStruct);

   /* Configure USART3_RX as alternate function */
   GPIO_InitStruct.GPIO_Mode = GPIO_Mode_AF;
   GPIO_InitStruct.GPIO_Pin  = GPIO_Pin_11;
   GPIO_Init(GPIOC, &GPIO_InitStruct);     


   /*
    * Init USART
    */
   USART_StructInit(&USART_InitStruct);    

   USART_InitStruct.USART_BaudRate            = 115200;
   USART_InitStruct.USART_WordLength          = USART_WordLength_8b;
   USART_InitStruct.USART_StopBits            = USART_StopBits_1;
   USART_InitStruct.USART_Parity              = USART_Parity_No;
   USART_InitStruct.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
   USART_InitStruct.USART_Mode                = USART_Mode_Rx | USART_Mode_Tx;

   /* USART configuration */
   USART_Init(USART3, &USART_InitStruct);
    
   /* Enable USART */
   USART_Cmd(USART3, ENABLE);

} /* USART3Init */

/*************************************************************************/
/*  HardwareInit                                                         */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
static void HardwareInit (void)
{  
   GPIO_InitTypeDef  GPIO_InitStruct;
 
   /* Turn on GPIOC clock for LED */
   RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);

   LD1_OFF();

   /* Set PC13 to output */
   GPIO_InitStruct.GPIO_Pin   = GPIO_Pin_13;
   GPIO_InitStruct.GPIO_Mode  = GPIO_Mode_OUT;
   GPIO_InitStruct.GPIO_OType = GPIO_OType_PP;
   GPIO_InitStruct.GPIO_Speed = GPIO_Speed_100MHz;
   GPIO_InitStruct.GPIO_PuPd  = GPIO_PuPd_NOPULL;
   GPIO_Init(GPIOC, &GPIO_InitStruct);
   
   /* Init USART 3 */
   USART3Init();   
   
   /* System configuration controller clock enable */   
   RCC->APB2ENR |= RCC_APB2ENR_SYSCFGEN;   
   
   /* Init SysTick */
   SystemCoreClockUpdate();   
   SysTick_Config(SystemCoreClock / TICKS_PER_SECOND);

} /* HardwareInit */

/*************************************************************************/
/*  LEDTask                                                              */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static msg_t LEDTask (void *p)
{
   (void)p;
   
   while(1)
   {
      LD1_ON();
      TimeDly(MS_2_TICKS(TASK_LED_DELAY_MS));
      LD1_OFF();
      TimeDly(MS_2_TICKS(TASK_LED_DELAY_MS));
   }
   
   return(0);
} /* LEDTask */

/*************************************************************************/
/*  HttpTask                                                             */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static msg_t HttpTask (void *p)
{
   (void)p;

   while(!IPIsNetIfUp())
   {
      TimeDly(MS_2_TICKS(500));
   }

   /* 
    * Initialize the TCP socket interface. 
    */
   StreamInit();

   /* 
    * Register media type defaults. These are configurable
    * in include/cfg/http.h or the Nut/OS Configurator.
    */
   MediaTypeInitDefaults();
   HttpRegisterRedir("", "/index.html", 301);  

   /* 
    * Wait for a client (browser) and handle its request.
    * This function will only return on unrecoverable errors. 
    */
   StreamClientAccept(HttpdClientHandler, NULL);
   
   /* Typically this point will be never reached. */
   while(1)
   {
      TimeDly(MS_2_TICKS(500));
   }
   
   return(0);        
} /* HttpTask */

/*************************************************************************/
/*  StartTask                                                            */
/*                                                                       */
/*  In    : task parameter                                               */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
static msg_t StartTask (void *p)
{
   (void)p;
   
   /*
    * The StartTask will be used to start all other tasks in the system.
    * At the end the priority will be set to "IDLE" priority.
    */
    
   /* Print some output */
   printf("\n");
   printf("uHTTP sample, STM32-E407\n");
   printf("Build " __DATE__ " " __TIME__ "\n\n");
   
   printf("IP-Address : %s\n", IP_DEFAULT_ADDRESS);
   printf("IP-Netmask : %s\n", IP_DEFAULT_NETMASK);
   printf("IP-Gateway : %s\n", IP_DEFAULT_GATEWAY);   
   printf("MAC-Address: %02X:%02X:%02X:%02X:%02X:%02X\n",
          IP_MAC_ADDRESS_0, IP_MAC_ADDRESS_1, IP_MAC_ADDRESS_2,
          IP_MAC_ADDRESS_3, IP_MAC_ADDRESS_4, IP_MAC_ADDRESS_5);
   
   /* Create the LED task */
   chThdCreateStatic(task_led_stack, sizeof(task_led_stack), TASK_LED_PRIORITY, LEDTask, NULL);
   
   /* Init the IP stack */
   IPInit();          

   /* Create the IPerf task */
   iperf_Start(); 

   /* Create the HTTP server task */
   chThdCreateStatic(task_http_stack, sizeof(task_http_stack), TASK_IP_HTTP_PRIORITY, HttpTask, NULL);
   
   chThdSetPriority(TASK_START_PRIORITY_IDLE);

   while (1)
   {
      TimeDly(MS_2_TICKS(1000));
   }

   return(0);
} /* StartTask */

/*=======================================================================*/
/*  All code exported                                                    */
/*=======================================================================*/

/*************************************************************************/
/*  main                                                                 */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: never                                                        */
/*************************************************************************/
int main (void)
{
   /*
    * Before the application which is using ChibiOS/RT can be started, 
    * the hardware of the target must be initialized. The CPU is still
    * initialized by the CrossWorks for ARM startup. But the timer tick
    * must be init too. Even the GPIO can be init here.
    */
   HardwareInit();

   /*
    * ChibiOS/RT initialization.
    */
   chSysInit();

   /*
    * The main function will start all other threads,
    * therefore we need the highest priority for the moment.   
    */
   chThdSetPriority(HIGHPRIO);
    
   /* Create the Start task */
   chThdCreateStatic(task_start_stack, sizeof(task_start_stack), TASK_START_PRIORITY, StartTask, NULL);
              
   /*
    * The main function is not needed anymore, 
    * therefore the priority is set to the lowest value.
    * This will start all other tasks too.
    */
   chThdSetPriority(LOWPRIO);
                 
   while (1)
   {
      TimeDly(MS_2_TICKS(500));
   }
   
   return(0);
} /* main */

/*************************************************************************/
/*  TimeDly                                                              */
/*                                                                       */
/*  In    : Ticks                                                        */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
void TimeDly (uint32_t Ticks)
{
   chThdSleep(Ticks);
} /* TimeDly */

/*************************************************************************/
/*  __putchar (used by printf and puts)                                  */
/*                                                                       */
/*  Send a character over the USART3 port.                               */
/*                                                                       */
/*  In    : none                                                         */
/*  Out   : none                                                         */
/*  Return: none                                                         */
/*************************************************************************/
int __putchar (int ch)
{
   /* Wait for "Transmission Complete" */
   while (USART_GetFlagStatus(USART3, USART_FLAG_TC) == RESET) {};

   /* Send data */
   USART_SendData(USART3, ch);
   
   return(ch);
} /* __putchar */

/*** EOF ***/